@@ -1,9 +1,10 @@
 import os
 import sys
 from collections import namedtuple
+from string import Formatter
 from types import FunctionType
 
-from ..utils.terminal import NON_TTY, FULL
+from ..utils.terminal import FULL, NON_TTY
 
 ERROR = object()  # represents a config value not accepted.
 
@@ -85,9 +86,26 @@ def _text_input_factory():
     return _input
 
 
+def _format_input_factory(allowed):
+    def _input(x):
+        if not isinstance(x, str):
+            return bool(x)
+        fvars = parser.parse(x)
+        if any(f[1] not in allowed for f in fvars):
+            # f is a tuple (literal_text, field_name, format_spec, conversion)
+            return ERROR
+        return x
+
+    # I want to accept only some field names, and pure text.
+    allowed = set(allowed.split() + [None])
+    parser = Formatter()
+    return _input
+
+
 Config = namedtuple('Config', 'title length spinner bar unknown force_tty disable manual '
-                              'enrich_print receipt_text monitor stats elapsed title_length '
-                              'spinner_length')
+                              'enrich_print receipt receipt_text monitor elapsed stats '
+                              'title_length spinner_length refresh_secs monitor_end '
+                              'elapsed_end stats_end ctrl_c')
 
 
 def create_config():
@@ -101,12 +119,18 @@ def create_config():
             disable=False,
             manual=False,
             enrich_print=True,
+            receipt=True,
             receipt_text=False,
             monitor=True,
-            stats=True,
             elapsed=True,
+            stats=True,
+            monitor_end=True,
+            elapsed_end=True,
+            stats_end=True,
             title_length=0,
             spinner_length=0,
+            refresh_secs=0,
+            ctrl_c=True,
         )
 
     def set_global(theme=None, **options):
@@ -162,12 +186,18 @@ def create_config():
             disable=_bool_input_factory(),
             manual=_bool_input_factory(),
             enrich_print=_bool_input_factory(),
+            receipt=_bool_input_factory(),
             receipt_text=_bool_input_factory(),
-            monitor=_bool_input_factory(),
-            stats=_bool_input_factory(),
-            elapsed=_bool_input_factory(),
+            monitor=_format_input_factory('count total percent'),
+            monitor_end=_format_input_factory('count total percent'),
+            elapsed=_format_input_factory('elapsed'),
+            elapsed_end=_format_input_factory('elapsed'),
+            stats=_format_input_factory('rate eta'),
+            stats_end=_format_input_factory('rate'),
             title_length=_int_input_factory(0, 100),
             spinner_length=_int_input_factory(0, 100),
+            refresh_secs=_int_input_factory(0, 60 * 60 * 24),  # maximum 24 hours.
+            ctrl_c=_bool_input_factory(),
             # title_effect=_enum_input_factory(),  # TODO someday.
         )
         assert all(k in validations for k in Config._fields)  # ensures all fields have validations.
